import java.io.*;

import antlr.CommonAST;
import antlr.collections.AST;
import antlr.DumpASTVisitor;
import antlr.RecognitionException;
import antlr.TokenStreamException;

class AttributeType implements Printable {

    String oid;
    String name;
    String description;
    String superclass;
    boolean obsolete;
    String equality;  // equality matching rule
    String slapdSyntax;
    String ldapV3Syntax;

    public String toLdapV3Format() {
        return "attributetype ( " + oid + "\n" 
            + ( ( name != null)? " NAME '" + name + "'" : "")
            + ( ( description != null)? " DESC " + description : "")
            + ( ( obsolete )?    " OBSOLETE " : "" )
            + ( ( superclass != null)?  " SUP " + superclass : "")
	    + ( ( equality != null)? " EQUALITY " + equality : "")
            + " ) ";   
    }

    /* according to the syntax 
       attribute <name> [<name2>] { bin | ces | cis | tel | dn } 
       (taken from man slapd.conf OpenLDAP 1.2)
    */
    public String toSlapdFormat() {
	return "# " +  ( ( description != null)? " DESC " + description : "")
	    + "\n attribute   " + name + "   " + slapdSyntax + "\n";
    }
        
    public void setName(String aName) { name = aName; }
    public void setDescription(String aDescr) { description = aDescr; }
    public void setObsolete(boolean obs) { obsolete = obs; }
    public void setOid(String aOid) { oid = aOid; }
    public void setSuperclass(String aClass) { superclass = aClass; }
    public void setEquality(String equalityOid) { equality = equalityOid; }
    public void setLdapV3Syntax(String syntax) { ldapV3Syntax = syntax; }
    public void setSlapdSyntax(String syntax) { slapdSyntax = syntax; }

    /** 
     * Computes apropriate values for ldapV3Syntax, Superclass and 
     * equality based on slapdSyntax.
     */
    public void convertSlapdToLdapv3() {
	if (slapdSyntax != null) {
	    if ("bin".equals(slapdSyntax)) {
		// Binary
		ldapV3Syntax = "1.3.6.1.4.1.1466.115.121.1.5";
	    } else if ("ces".equals(slapdSyntax)) {
		// IA5 String
		ldapV3Syntax = "1.3.6.1.4.1.1466.115.121.1.26";
		// caseExactIA5Match
		equality = "1.3.6.1.4.1.1466.109.114.1";
	    } else if ("cis".equals(slapdSyntax)) {
		superclass = "name";
	    } else if ("tel".equals(slapdSyntax)) {
		superclass = "telephoneNumber";
	    } else if ("dn".equals(slapdSyntax)) {
		superclass = "distinguishedName";
	    }
	}
    }

    /**
     *
     */
    public void convertLdapV3ToSlapd() {
	if (ldapV3Syntax != null) {
	    // Binary
	    if ("1.3.6.1.4.1.1466.115.121.1.5".equals(ldapV3Syntax)) {
		slapdSyntax = "bin";
		return;
	    }
	    // IA5 String
	    if ("1.3.6.1.4.1.1466.115.121.1.26".equals(ldapV3Syntax)) {
		if (equality != null) {
		    if ("1.3.6.1.4.1.1466.109.114.2".equals(equality)
			|| "caseIgnoreIA5Match".equals(equality)) {
			slapdSyntax = "cis";
			return;
		    } 
		}
		slapdSyntax = "ces";
		return;
	    }
	    // telephoneNumber
	    if ("1.3.6.1.4.1.1466.115.121.1.50".equals(ldapV3Syntax)) {
		slapdSyntax = "tel";
		return;
	    }
	}

	// supertype == name
	if (superclass != null) {
	    if ("name".equals(superclass)
		|| "2.5.4.41".equals(superclass)) {
		slapdSyntax = "cis";
		return;
	    }
	    if ("telephoneNumber".equals(superclass)
		|| "2.5.4.20".equals(superclass)) {
		slapdSyntax = "tel";
		return;
	    }
	    if ("distinguishedName".equals(superclass)
		|| "2.5.4.49".equals(superclass)) {
		slapdSyntax = "dn";
		return;
	    }
	}
	
	// by default return cis if we don't have a guess
	slapdSyntax = "cis";
    }
}


