header { 
    import java.util.Vector;
    import java.util.HashMap;
}

/** 
 * This a grammar for LDAPv3 schema definitions
 * inspired from 
 *  "Understanding And Deploying LDAP Directory Services"
 *   Timothy A Howes, Ph.D. Mark C. Smith, Gordon S. Good
 *   Macmillan Technical Publishing, USA, 1999
 *   respectively rfc 2252
*/

class LDAPv3SchemaParser extends Parser ;
options {
    buildAST = true;
    k = 20;
    //exportVocab=LDAPv3Schema;
}

schema returns [java.util.Vector vec]
  { 
    vec = new java.util.Vector();
    AttributeType at;
    ObjectClass   oc;
    
  }
    : (at=attributetype 
         {
           vec.add(at);
         }
      | oc=objectclass
         {
           vec.add(oc);
         } 
      )+
    ;


/*
      AttributeTypeDescription = "(" whsp
            numericoid whsp              ; AttributeType identifier
          [ "NAME" qdescrs ]             ; name used in AttributeType
          [ "DESC" qdstring ]            ; description
          [ "OBSOLETE" whsp ]
          [ "SUP" woid ]                 ; derived from this other
                                         ; AttributeType
          [ "EQUALITY" woid              ; Matching Rule name
          [ "ORDERING" woid              ; Matching Rule name
          [ "SUBSTR" woid ]              ; Matching Rule name
          [ "SYNTAX" whsp noidlen whsp ] ; see section 4.3
          [ "SINGLE-VALUE" whsp ]        ; default multi-valued
          [ "COLLECTIVE" whsp ]          ; default not collective      AttributeTypeDescription = "(" whsp
            numericoid whsp              ; AttributeType identifier
          [ "NAME" qdescrs ]             ; name used in AttributeType
          [ "DESC" qdstring ]            ; description
          [ "OBSOLETE" whsp ]
          [ "SUP" woid ]                 ; derived from this other
                                         ; AttributeType
          [ "EQUALITY" woid              ; Matching Rule name
          [ "ORDERING" woid              ; Matching Rule name
          [ "SUBSTR" woid ]              ; Matching Rule name
          [ "SYNTAX" whsp noidlen whsp ] ; see section 4.3
          [ "SINGLE-VALUE" whsp ]        ; default multi-valued
          [ "COLLECTIVE" whsp ]          ; default not collective
          [ "NO-USER-MODIFICATION" whsp ]; default user modifiable
          [ "USAGE" whsp AttributeUsage ]; default userApplications
          whsp ")"

      AttributeUsage =
          "userApplications"     /
          "directoryOperation"   /
          "distributedOperation" / ; DSA-shared
          "dSAOperation"          ; DSA-specific, value depends on server


          [ "NO-USER-MODIFICATION" whsp ]; default user modifiable
          [ "USAGE" whsp AttributeUsage ]; default userApplications
          whsp ")"

      AttributeUsage =
          "userApplications"     /
          "directoryOperation"   /
          "distributedOperation" / ; DSA-shared
          "dSAOperation"          ; DSA-specific, value depends on server
 */
attributetype returns [AttributeType at]
  { 
    at=new AttributeType(); 
    String supOid=null;
    String eqOid=null;
    String orderingOid=null;
    String substrOid=null;
  }
    :    "attributetype" LPAREN 
           (oid:NUMERICOID)? 
            ( "NAME" name:QIDENT )?
            ( obsolete:"OBSOLETE" )?
            ( desc:DESCR )?
            ( "SUP"  supOid=oid )?
            ( "EQUALITY" eqOid=oid )?
            ( "ORDERING" orderingOid=oid )?  
            ( "SUBSTR" substrOid=oid )?  
            ( "SYNTAX" )?
            ( "SINGLE-VALUE" )?
            ( "COLLECTIVE" )?
            ( "NO-USER-MODIFICATION" )?
            ( "USAGE" )?
         RPAREN
          {  
	     at.setOid(oid.getText());
             if (name != null) at.setName( name.getText());
             if (desc != null) at.setDescription( desc.getText());
             if (obsolete != null) at.setObsolete( true );
             //System.out.println("\n" + at.toLdapV3Format() + "\n");
          }
    ;


/* from rfc2252   
      ObjectClassDescription = "(" whsp
          numericoid whsp      ; ObjectClass identifier
          [ "NAME" qdescrs ]
          [ "DESC" qdstring ]
          [ "OBSOLETE" whsp ]
          [ "SUP" oids ]       ; Superior ObjectClasses
          [ ( "ABSTRACT" / "STRUCTURAL" / "AUXILIARY" ) whsp ]
                               ; default structural
          [ "MUST" oids ]      ; AttributeTypes
          [ "MAY" oids ]       ; AttributeTypes
      whsp ")"
*/

objectclass returns [ObjectClass oc]
    {
	oc = new ObjectClass();
	Vector supOids = null;
	Vector mustOids = null;
	Vector mayOids = null;
    }
    : ("objectclass" | "objectClass" )  LPAREN
        oid:NUMERICOID
	( "NAME" name:QIDENT )?
        ( desc:DESCR )?
	( obsolete:"OBSOLETE" )?
        ( "SUP" supOids=oids )?
        ( "MUST" mustOids=oids )?
        ( "MAY" mayOids=oids )?
      RPAREN
      {
	  oc.setOid(oid.getText());
	  if (name != null) oc.setName(name.getText());
	  oc.setSuperclasses (supOids);
	  oc.setRequiredAttributes (mustOids);
	  oc.setOptionalAttributes (mayOids);
      }
    ;


oids returns [Vector oids]
  {
      oids = new Vector();
      String oid = null;
  }
    : (oid=oid { oids.add(oid); })
    | LPAREN oid=oid { oids.add(oid); } ( DOLLAR oid=oid { oids.add(oid);} )+ RPAREN  
    ;


oid returns [String oid]
  {
    oid = null;
  }  
    : tokn:NUMERICOID 
      {
	  oid = tokn.getText();
      }
    | toki:IDENT
      {
	  oid = toki.getText();
      }
    ;


/*  LEXICAL RULES  */
class LDAPv3SchemaLexer extends Lexer;
options {
	k=10;
        charVocabulary = '\3'..'\377'; 
	//exportVocab=LDAPv3Schema;
   //analyzerDebug = true;
   //codeGenDebug = true;
}


DOLLAR : '$';

//ATTRIBUTETYPE
//    : "attributetype"
//    ;

WS	
options {
  paraphrase = "white space";
}
	:	(' '
	|	'\t'
	|	'\n'  { newline(); }
	|	'\r')
		{ $setType(Token.SKIP); }
	;

COMMENT
	: '#' (~'\n')* '\n' { newline(); $setType(Token.SKIP); }
	;

DOT
    : '.'
    ;

LPAREN:	'('
	;

RPAREN:	')'
	;


DESCR : "DESC" (WS)* (descr:QSTRING) 
  { // remove "DESC" and whitespace in the returned token
    $setText(descr.getText());
  };

protected
QSTRING
options {
  testLiterals = true;
  paraphrase = "a single quoted string";
}
    :	'\'' (~'\'')* '\''
    ;

QIDENT
options {
  paraphrase = "a quoted identifier";
}
	:	"'" (ident:IDENT) "'"
	{ $setText(ident.getText()); }
	;

IDENT
options {
  testLiterals = true;
  paraphrase = "an identifer";
}
	:	('a'..'z'|'A'..'Z'|'_')('a'..'z'|'A'..'Z'|'_'|'0'..'9')* 
	;


protected
DIGIT
options {
  paraphrase = "a digit";
}
	:	'0'..'9'
	;

//protected
NUMERICOID
    //    : INT (DOT INT)*
    : ('1'..'9')('0'..'9')* (DOT ('1'..'9')('0'..'9')*)*
    ;

/*
WNUMERICOID
    : (WS)? noid:NUMERICOID
      {   // remove the whitespace from this token
	  $setText(noid.getText());
      }
    ;
*/

protected
INT 
    : (DIGIT)+
    ;


